
#ifndef _XSECPROTO_H_
#define _XSECPROTO_H_

#if SEC_BASE

/*
 * Copyright (c) SecureWare, Inc, 1994, All rights reserved.
 *
 * @(#)04 1.5 x11r5/extensions/include/xsecurity/Xsecproto.h, xsecext, sco_3.0, 301FCS 14:26:45 1/2/95, SecureWare, Inc.
 */

#include <X11/extensions/Xsecdefs.h>
#include <X11/Xmd.h>

/*
 * For the purpose of the structure definitions in this file, we must
 * redefine the following types in terms of Xmd.h's types.  All of these
 * are #undef'd at the end of this file, restoring the definitions in X.h.
*/

#define Window CARD32
#define Drawable CARD32
#define Font CARD32
#define Pixmap CARD32
#define Cursor CARD32
#define Colormap CARD32
#define GContext CARD32
#define Atom CARD32
#define VisualID CARD32
#define Time CARD32

/*
 * Security Extension Protocol Request Minor Opcodes
 */
#define	X_AllocReservedColor		1
#define	X_AllocSecurityStripe		2
#define	X_AllowUntrustedClients		3
#define	X_SelectSecurityEvents		4
#define	X_RegisterService		5
#define	X_UnregisterService		6
#define	X_QueryServiceRegistry		7
#define	X_SetInactivityTimeout		8
#define	X_GetInactivityTimeout		9
#define X_KillAllClients		10
#define X_CopyProperty			11
#define X_GetClientAttributes		12
#define X_SetEffectivePrivileges	13
#define X_ComposePropertyAtom		14
#define X_DecomposePropertyAtom		15
#define X_SetMultilevelPropertyMode	16
#define X_GetMultilevelPropertyMode	17
#define X_SetSessionClearance		18
#define X_GetObjectAttributes		19
#define X_SetObjectAttributes		20
#define X_RestrictGrabs			21
#define X_AllowGrabs			22
#define	X_OverrideGrabButton		23
#define	X_OverrideGrabKey		24
#define	X_OverrideGrabKeyboard		25
#define	X_OverrideGrabPointer		26
#define	X_GetDefaultPermissions		27
#define	X_SetDefaultPermissions		28
#define X_GetDisplayAcl			29
#define X_SetDisplayAcl			30
#define X_SetScreenLock			31
#define X_ClearResetWindow		32
#define X_ListenToTrustedClient		33
#define X_SwitchScreen			34

/*
 * Miscellaneous protocol structures
 */
typedef struct {
    Atom serviceName B32;
    Window serviceWindow B32;
} xRegistrationEntry;

typedef struct {
    Atom attrType B32;		/* Attribute type represented */
    CARD32 attrLen B32;		/* Length in 32-bit words */
} xAttribute;

typedef struct {
    Atom objectType B32;	/* Object type represented */
    CARD32 perms B32;		/* Permissions value */
} xPermissions;

/* ACL structure for LIST of ACL embedded in requests/replies */

#define sz_xDisplayAcl 16

typedef struct {
    INT32 family B32;
    CARD32 length B32;
    CARD32 user B32;
    CARD32 access B32;
} xDisplayAcl;

/*
 * Request Formats
 */

#define	sz_xAllocReservedColorReq	16

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Colormap cmap B32;
    CARD16 red B16, green B16, blue B16;
    CARD16 pad B16;
} xAllocReservedColorReq;


#define	sz_xAllocSecurityStripeReq	16

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Window root B32;
    Window window B32;
    CARD16 height B16;
    BYTE pad[2];
} xAllocSecurityStripeReq;


#define	sz_xSelectSecurityEventsReq	16

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Window window B32;
    CARD32 mask B32; 
    CARD32 mask2 B32;
} xSelectSecurityEventsReq;


#define	sz_xRegisterServiceReq	20

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Window window B32;
    Atom serviceName B32;
    Window serviceWindow B32;
    BOOL exitFlag;
    BYTE pad[3];
} xRegisterServiceReq;


#define	sz_xUnregisterServiceReq	12

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Window window B32;
    Atom serviceName B32;
} xUnregisterServiceReq;


#define	sz_xQueryServiceRegistryReq	12

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Window window B32;
    Atom serviceName B32;
} xQueryServiceRegistryReq;


#define	sz_xSetInactivityTimeoutReq 8

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    CARD32 timeout B32;
} xSetInactivityTimeoutReq;


#define sz_xCopyPropertyReq 24

typedef struct {
    CARD8 reqType;
    BYTE secReqType;	/* Always X_CopyProperty */
    CARD16 length B16;
    Window srcWin B32, dstWin B32;
    Atom srcProp B32, dstProp B32;
    CARD8 mode;
    BOOL delete;
    BYTE pad[2];
} xCopyPropertyReq;


#define sz_xGetClientAttributesReq 8

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    XID clientID B32;
} xGetClientAttributesReq;


#define	sz_xSetEffectivePrivilegesReq 12

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    CARD32 priv1 B32;
    CARD32 priv2 B32;
} xSetEffectivePrivilegesReq;


#define sz_xComposePropertyAtomReq	8

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Atom atom B32;
} xComposePropertyAtomReq;


/* Shares structure with ComposePropertyAtom */

#define sz_xDecomposePropertyAtomReq sz_xComposePropertyAtomReq

typedef xComposePropertyAtomReq xDecomposePropertyAtomReq;


#define sz_xSetMultilevelPropertyModeReq	8

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    BYTE composeMode;
    BYTE labelType;
    CARD16 pad1 B16;
} xSetMultilevelPropertyModeReq;


#define sz_xGetObjectAttributesReq 20

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    XID objID1 B32;		/* The objIDx fields identify the object */
    XID objID2 B32;
    XID objID3 B32;
    Atom objType B32;		/* Which kind of object. */
} xGetObjectAttributesReq;


/* Shares structure with GetObjectAttributes */

#define sz_xSetObjectAttributesReq sz_xGetObjectAttributesReq

typedef xGetObjectAttributesReq xSetObjectAttributesReq;


#define sz_xOverrideGrabButtonReq 24

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Window grabWindow B32;
    CARD16 eventMask B16;
    BYTE pointerMode, keyboardMode;
    Window confineTo B32;
    Cursor cursor B32;
    CARD8 button;
    BOOL ownerEvents;
    CARD16 modifiers B16;
} xOverrideGrabButtonReq;


#define sz_xOverrideGrabKeyReq 16

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Window grabWindow B32;
    CARD16 modifiers B16;
    CARD8 key;
    BYTE pointerMode, keyboardMode;  
    BOOL ownerEvents;
    BYTE pad1, pad2;
} xOverrideGrabKeyReq;


#define sz_xOverrideGrabKeyboardReq 16

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Window grabWindow B32;
    Time time B32;
    BYTE pointerMode, keyboardMode;  
    BOOL ownerEvents;
    BYTE pad;
} xOverrideGrabKeyboardReq;


#define sz_xOverrideGrabPointerReq 28

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    Window grabWindow B32;
    CARD16 eventMask B16;
    BYTE pointerMode, keyboardMode;
    Window confineTo B32;
    Cursor cursor B32;
    Time time B32;
    BOOL ownerEvents;
    BYTE pad1, pad2, pad3;
} xOverrideGrabPointerReq;


#define sz_xGetDefaultPermissionsReq 8

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    BYTE whichSet;
    BYTE pad1, pad2, pad3;
} xGetDefaultPermissionsReq;


#define sz_xSetDefaultPermissionsReq 8

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    BYTE whichSet;
    BYTE pad1;
    CARD16 nPermissions B16;
} xSetDefaultPermissionsReq;


#define sz_xSetDisplayAclReq 12

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    CARD32 aclLength B32;
    CARD32 aclEntries B32;
} xSetDisplayAclReq;


#define sz_xSetScreenLockReq 8

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    BOOL lockScreen;
    BYTE pad1;
    CARD16 pad2 B16;
} xSetScreenLockReq;


#define sz_xListenToTrustedClientReq 8

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    BOOL listenToClient;
    BYTE pad1;
    CARD16 pad2 B16;
} xListenToTrustedClientReq;


#define sz_xSwitchScreenReq 8

typedef struct {
    CARD8 reqType;
    BYTE secReqType;
    CARD16 length B16;
    CARD32 screen B32;
} xSwitchScreenReq;


#define sz_xClearResetWindowReq 8

typedef struct {
    CARD8 reqType;
    BYTE secReqType;	/* Always X_ClearResetWindow */
    CARD16 length B16;
    Window win B32;
} xClearResetWindowReq;


/*
 * Reply formats.
 */

#define	sz_xAllocReservedColorReply	32

typedef struct {
    BYTE type; /* X_Reply */
    BYTE pad1;
    CARD16 sequenceNumber B16;
    CARD32 length B32;   /* 0 */
    CARD16 red B16, green B16, blue B16;
    CARD16 pad2 B16;
    CARD32 pixel B32;
    CARD32 pad3 B32;
    CARD32 pad4 B32;
    CARD32 pad5 B32;
    } xAllocReservedColorReply;


#define	sz_xQueryServiceRegistryReply	32

typedef struct {
    BYTE type; /* X_Reply */
    BYTE pad1;
    CARD16 sequenceNumber B16;
    CARD32 length B32;
    CARD16 nServices B16;
    CARD16 pad2 B16;
    CARD32 pad3 B32;
    CARD32 pad4 B32;
    CARD32 pad5 B32;
    CARD32 pad6 B32;
    CARD32 pad7 B32;
    } xQueryServiceRegistryReply;

#define	sz_xGetInactivityTimeoutReply 32

typedef struct {
    BYTE type;  /* X_Reply */
    CARD8 pad1;
    CARD16 sequenceNumber B16;
    CARD32 length B32;
    CARD32 timeout B32;
    CARD32 timeleft B32;
    CARD32 pad2 B32;
    CARD32 pad3 B32;
    CARD32 pad4 B32;
    CARD32 pad5 B32;
} xGetInactivityTimeoutReply;


#define	sz_xGetClientAttributesReply	32

typedef struct {
    BYTE type; /* X_Reply */
    BYTE pad1;
    CARD16 sequenceNumber B16;
    CARD32 length B32;
    CARD32 nAttributes B32;
    CARD32 pad2 B32;
    CARD32 pad3 B32;
    CARD32 pad4 B32;
    CARD32 pad5 B32;
    CARD32 pad6 B32;
    } xGetClientAttributesReply;


#define	sz_xComposePropertyAtomReply 32

typedef struct {
    BYTE type;  /* X_Reply */
    CARD8 pad1;
    CARD16 sequenceNumber B16;
    CARD32 length B32;
    Atom atom B32;
    CARD32 pad2 B32;
    CARD32 pad3 B32;
    CARD32 pad4 B32;
    CARD32 pad5 B32;
    CARD32 pad6 B32;
} xComposePropertyAtomReply;


#define	sz_xDecomposePropertyAtomReply 32

/* Shares structure with ComposePropertyAtom */
typedef xComposePropertyAtomReply xDecomposePropertyAtomReply;


#define	sz_xGetMultilevelPropertyModeReply 32

typedef struct {
    BYTE type;  /* X_Reply */
    CARD8 pad1;
    CARD16 sequenceNumber B16;
    CARD32 length B32;
    BYTE composeMode;
    BYTE labelType;
    CARD16 pad2 B16;
    CARD32 pad3 B32;
    CARD32 pad4 B32;
    CARD32 pad5 B32;
    CARD32 pad6 B32;
    CARD32 pad7 B32;
} xGetMultilevelPropertyModeReply;


/* Shares reply structure with GetClientAttributes */

#define	sz_xGetObjectAttributesReply	sz_xGetClientAttributesReply

typedef xGetClientAttributesReply xGetObjectAttributesReply;


#define	sz_xGetDefaultPermissionsReply	32

typedef struct {
    BYTE type; /* X_Reply */
    BYTE pad1;
    CARD16 sequenceNumber B16;
    CARD32 length B32;
    CARD32 nPermissions B32;
    CARD32 pad2 B32;
    CARD32 pad3 B32;
    CARD32 pad4 B32;
    CARD32 pad5 B32;
    CARD32 pad6 B32;
} xGetDefaultPermissionsReply;


#define sz_XGetDisplayAclReply 32

typedef struct {
    BYTE type;  /* X_Reply */
    CARD8 pad1;
    CARD16 sequenceNumber B16;
    CARD32 length B32;
    CARD32 aclLength B32;
    CARD32 aclEntries B32;
    CARD32 enabled B32;
    CARD32 pad2 B32;
    CARD32 pad3 B32;
    CARD32 pad4 B32;
} xGetDisplayAclReply;


#define	sz_xSwitchScreenReply 32

typedef struct {
    BYTE type;  /* X_Reply */
    CARD8 pad1;
    CARD16 sequenceNumber B16;
    CARD32 length B32;
    CARD32 status B32;
    CARD32 pad2 B32;
    CARD32 pad3 B32;
    CARD32 pad4 B32;
    CARD32 pad5 B32;
    CARD32 pad6 B32;
} xSwitchScreenReply;


/*
 * Event Formats 
 */

typedef struct {
    union {
	struct {
	    BYTE type;
	    BYTE detail;
	    CARD16 sequenceNumber B16;
	    Window window B32;
	    CARD32 pad1 B32;
	    CARD32 pad2 B32;
	    CARD32 pad3 B32;
	    CARD32 pad4 B32;
	    CARD32 pad5 B32;
	    CARD32 pad6 B32;
	    } u;
	struct {
            CARD32 pad0 B32;
            CARD32 pad1 B32;	/* Registration Window */
	    Atom service B32;	/* Service name */
	    } service;
	struct {
            CARD32 pad0 B32;
            CARD32 pad1 B32;	/* Root Window */
	    Window requestor B32;
	    Atom selection B32, target B32, property B32;
            Time time B32;    
	    XID clientID B32;
	    } selectionRedirect;
	struct {
            CARD32 pad0 B32;
            CARD32 pad1 B32;	/* Window from ConvertSelection */
	    Window owner B32;
	    Atom selection B32;
            Time time B32;    
	    XID clientID B32;
            CARD32 pad2 B32;
            CARD32 pad3 B32;
	    } selectionForward;
	struct {
	    CARD32 pad0 B32;
	    CARD32 pad1 B32;	/* Registration Window */
	    Atom attrType B32;	/* Attribute type name */
	    } attribute;
	struct {
	    CARD32 pad0 B32;
	    CARD32 pad1 B32;	/* Root Window */
	    BOOL acquired;	/* Screen acquired or released */
	    } screen;
    } u;
} xSecEvent;

/* restore these definitions back to the typedefs in X.h */
#undef Window
#undef Drawable
#undef Font
#undef Pixmap
#undef Cursor
#undef Colormap
#undef GContext
#undef Atom
#undef VisualID
#undef Time

#endif	/* SEC_BASE */
#endif /* _XSECPROTO_H */
